package com.huawei.dli.jdbc;

import com.huawei.dli.jdbc.utils.ResultFetchingSession;

import java.sql.ResultSet;
import java.sql.SQLException;

public class DliForwardResultSet extends DliResultSet {

    private ResultFetchingSession session;

    private final long totalRows;

    private boolean isClosed = false;

    private Object[] currentRow;

    public DliForwardResultSet(DliStatement stmt, ResultFetchingSession session) throws SQLException {
        super(stmt.getConnection(), stmt, session.getSchema());
        this.session = session;
        int maxRows = stmt.getMaxRows();
        long recordCount = session.getTotalCount();
        if (maxRows >= 0 && maxRows <= recordCount) {
            this.totalRows = maxRows;
        } else {
            this.totalRows = session.getTotalCount();
        }
    }

    protected void checkClosed() throws SQLException {
        if (isClosed) {
            throw new SQLException("The result set has been closed");
        }
    }

    @Override
    public int getRow() throws SQLException {
        checkClosed();
        return session.getFetchedRows();
    }

    @Override
    public int getType() throws SQLException {
        return ResultSet.TYPE_FORWARD_ONLY;
    }

    @Override
    public boolean isClosed() throws SQLException {
        return isClosed;
    }

    @Override
    Object[] rowAtCursor() throws SQLException {
        if (currentRow == null) {
            throw new SQLException("the row should be not-null, row=" + session.getFetchedRows());
        }

        if (currentRow.length == 0) {
            throw new SQLException("the row should have more than 1 column , row=" + session.getFetchedRows());
        }

        return currentRow;
    }

    @Override
    public boolean next() throws SQLException {
        checkClosed();
        if (session.getFetchedRows() == totalRows || session.getFetchedRows() == -1) {
            close();
            return false;
        }

        currentRow = session.readNextRecord();
        return true;
    }

    @Override
    public void close() throws SQLException {
        if (isClosed) {
            return;
        }
        currentRow = null;
        session.close();
        session = null;
        isClosed = true;
    }
}
