package com.huawei.dli.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Iterator;

public class DliStaticResultSet extends DliResultSet implements ResultSet {

    private Iterator<Object[]> iterator;

    Object[] row;

    /**
     * For some meta query (like procedures) we need to return an empty query result.
     */
    private boolean isEmptyResultSet = false;

    DliStaticResultSet(DliConnection conn, DliResultSetMetaData meta) throws SQLException {
        super(conn, null, meta);
        isEmptyResultSet = true;
    }

    DliStaticResultSet(DliConnection conn, DliResultSetMetaData meta, Iterator<Object[]> iter)
        throws SQLException {
        super(conn, null, meta);
        iterator = iter;
        isEmptyResultSet = false;
    }

    @Override
    Object[] rowAtCursor() throws SQLException {
        return row;
    }

    @Override
    public boolean next() throws SQLException {
        if (isEmptyResultSet) {
            return false;
        }
        if (iterator.hasNext()) {
            row = iterator.next();
            return true;
        } else {
            return false;
        }
    }

    @Override
    public void close() throws SQLException {
        iterator = null;
    }
}
