package com.huawei.dli.jdbc.utils;

import com.huawei.dli.sdk.Reader;
import com.huawei.dli.sdk.common.Row;
import com.huawei.dli.sdk.exception.DLIException;

import java.io.IOException;

/**
 * Use single reader to read data from OBS, no use buffer,
 * in this scenario, the order of the result data is limited by the list of returned files.
 */
public class CommonIterator implements DownloadIterator {
    private final Reader reader;

    private final long resultCount;

    public CommonIterator(Reader reader, long totalCount) {
        this.reader = reader;
        this.resultCount = totalCount;
    }

    public CommonIterator(Reader reader) {
        this.reader = reader;
        this.resultCount = Long.MAX_VALUE;
    }

    @Override
    public boolean hasNext() {
        return reader.hasNext();
    }

    @Override
    public Row fetch() throws IOException {
        if (hasNext()) {
            try {
                return reader.read();
            } catch (DLIException | IOException e) {
                throw new IOException("Fetch data failed", e);
            }
        }
        throw new RuntimeException("There is no new data to fetch, please check");
    }

    @Override
    public void close() throws IOException {
        reader.close();
    }

    @Override
    public long getResultCount() {
        return resultCount;
    }
}
