package com.huawei.dli.jdbc.utils;

import com.huawei.dli.sdk.common.JobType;
import com.huawei.dli.sdk.common.Row;
import com.huawei.dli.sdk.common.TableSchema;

import java.util.List;

import lombok.Getter;

@Getter
public class ExecuteJobResponse {

    private final boolean isSuccess;

    private String message;

    private String jobId;

    private JobType jobType;

    private TableSchema schema;

    private List<Row> rows;

    private ResultFetchingSession resultFetchingSession;

    private String jobMode;

    private boolean hasResultSet = false;

    public ExecuteJobResponse(boolean isSuccess, String message) {
        this.isSuccess = isSuccess;
        this.message = message;
    }

    public ExecuteJobResponse(
        String jobId,
        JobType jobType,
        String jobMode,
        TableSchema resultSchema,
        List<Row> rows) throws DliException {
        this.isSuccess = true;
        this.jobId = jobId;
        this.jobType = jobType;
        this.jobMode = jobMode;
        this.schema = resultSchema;
        this.rows = rows;
        if (this.schema != null && rows != null) {
            this.hasResultSet = true;
            resultFetchingSession = new SchemaResultFetchingSession(schema, rows);
        }
    }

    public boolean hasResultSet() {
        return hasResultSet;
    }

    // If the response json body form server has the rows element, this sql is execute in Sync Mode.
    // If the sql fetch no records, the rows may empty but not null.
    public boolean isSyncMode() {
        return (jobMode.equals("sync"));
    }
}
