package com.huawei.dli.jdbc.utils;

import com.huawei.dli.jdbc.DliResultSetMetaData;
import com.huawei.dli.sdk.common.Row;
import com.huawei.dli.sdk.common.TableSchema;

import java.io.IOException;

public class QueryExportFetchingSession extends ResultFetchingSession {
    private final DownloadIterator innerIter;

    private int fetchedRows;

    private boolean isClosed = false;

    private DliResultSetMetaData schema;

    public QueryExportFetchingSession(String jobId, IDliServiceProxy serviceProxy,
        TableSchema schemaOrigin) throws DliException {
        schema = resolveSchema(schemaOrigin != null ? schemaOrigin : serviceProxy.getResultSchema(jobId));
        innerIter = serviceProxy.exportJobResult(jobId);
        this.fetchedRows = 0;
    }

    public QueryExportFetchingSession(String jobId, IDliServiceProxy serviceProxy) throws DliException {
        this(jobId, serviceProxy, null);
    }

    @Override
    public Object[] readNextRecord() throws DliException {
        if (isClosed) {
            throw new DliException("Query result fetching session has been closed",
                ErrorCode.JDBC_CONNECTION_ALREADY_CLOSED_ERROR.toString());
        }
        try {
            Row row = innerIter.fetch();
            if (row == null) {
                throw new DliException("The Row is null when readNextRecord",
                        ErrorCode.JDBC_DLI_SERVER_ERROR.toString());
            }
            fetchedRows++;
            return row.getCols().toArray();
        } catch (IOException e) {
            throw new DliException(e);
        }
    }

    @Override
    public long getTotalCount() {
        if (this.innerIter != null) {
            return this.innerIter.getResultCount();
        }
        return -1;
    }

    @Override
    public int getFetchedRows() {
        if (!innerIter.hasNext()) {
            return -1;
        }
        return fetchedRows;
    }

    @Override
    public DliResultSetMetaData getSchema() {
        return schema;
    }

    @Override
    public void close() {
        schema = null;
        try {
            innerIter.close();
        } catch (IOException e) {
            // ignore it
        }
        fetchedRows = 0;
        isClosed = true;
    }
}
