package com.huawei.dli.jdbc.utils;

import com.huawei.dli.jdbc.DliResultSetMetaData;
import com.huawei.dli.sdk.common.TableSchema;

public class QueryResultFetchingSession extends ResultFetchingSession {
    private int currentPage;

    private int pageSize;

    private int fetchedRows;

    private String jobId;

    private GetJobResultResponse resultBuffer;

    IDliServiceProxy serviceProxy;

    private boolean isClosed = false;

    DliResultSetMetaData schema;

    public QueryResultFetchingSession(String jobId, IDliServiceProxy serviceProxy, int bufferSize)
        throws DliException {
        this(jobId, serviceProxy, bufferSize, null);
    }

    public QueryResultFetchingSession(String jobId, IDliServiceProxy serviceProxy, int bufferSize,
        TableSchema schemaOrigin) throws DliException {
        this.jobId = jobId;
        this.serviceProxy = serviceProxy;
        this.currentPage = 1;
        this.pageSize = bufferSize;
        fetchedRows = 0;
        schema = resolveSchema(schemaOrigin != null ? schemaOrigin : serviceProxy.getResultSchema(jobId));
        resultBuffer = serviceProxy.getJobResult(jobId, pageSize, currentPage);
    }

    public QueryResultFetchingSession(String jobId, IDliServiceProxy serviceProxy,
        TableSchema schema) throws DliException {
        this(jobId, serviceProxy, ConnectionResource.RESULT_SET_BUFFER_SIZE, schema);
    }

    public Object[] readNextRecord() throws DliException {
        if (isClosed) {
            throw new DliException("Query result fetching session has been closed",
                ErrorCode.JDBC_CONNECTION_ALREADY_CLOSED_ERROR.toString());
        }
        int index = fetchedRows % pageSize;
        int pageIndex = fetchedRows / pageSize + 1;
        if (pageIndex != currentPage) {
            currentPage = pageIndex;
            resultBuffer = serviceProxy.getJobResult(jobId, pageSize, currentPage);
        }

        fetchedRows++;
        return resultBuffer.getRows().get(index).getCols().toArray();
    }

    public long getTotalCount() {
        if (resultBuffer != null) {
            return resultBuffer.getRowCount();
        } else {
            return -1;
        }
    }

    public int getFetchedRows() {
        return fetchedRows;
    }

    public DliResultSetMetaData getSchema() {
        return schema;
    }

    public void close() {
        resultBuffer = null;
        schema = null;
        currentPage = 0;
        pageSize = 0;
        fetchedRows = 0;
        jobId = null;
        serviceProxy = null;
        isClosed = true;
    }
}
