package com.huawei.dli.jdbc.utils;

import com.huawei.dli.jdbc.DliResultSetMetaData;
import com.huawei.dli.sdk.common.Row;
import com.huawei.dli.sdk.common.TableSchema;

import java.util.List;

public class SchemaResultFetchingSession extends ResultFetchingSession {

    private List<Row> rows;

    private boolean isClosed = false;

    private int fetchedRows;

    DliResultSetMetaData schema;

    public SchemaResultFetchingSession(TableSchema schema, List<Row> rows) throws DliException {
        this.schema = resolveSchema(schema);
        this.rows = rows;
        fetchedRows = 0;
    }

    @Override
    public Object[] readNextRecord() throws DliException {
        if (isClosed) {
            throw new DliException("Query result fetching session has been closed",
                ErrorCode.JDBC_DLI_CONNECTION_ERROR.toString());
        }
        return rows.get(fetchedRows++).getCols().toArray();
    }

    @Override
    public long getTotalCount() {
        return rows.size();
    }

    @Override
    public int getFetchedRows() {
        return fetchedRows;
    }

    @Override
    public DliResultSetMetaData getSchema() {
        return schema;
    }

    @Override
    public void close() {
        schema = null;
        rows = null;
        fetchedRows = 0;
        isClosed = true;
    }
}
