package com.huawei.dli.jdbc.utils.type;

public abstract class AbstractChar<T extends AbstractChar> {
    protected String value;

    AbstractChar(String value) {
        this(value, value.length());
    }

    AbstractChar(String value, int maxLength) {
        this.value = this.enforceMaxLength(value, maxLength);
    }

    private String enforceMaxLength(String val, int maxLength) {
        if (val == null) {
            return null;
        } else {
            String valueInfo = val;
            if (maxLength > 0) {
                int valLength = val.codePointCount(0, val.length());
                if (valLength > maxLength) {
                    valueInfo = val.substring(0, val.offsetByCodePoints(0, maxLength));
                }
            }

            return valueInfo;
        }
    }

    public String getValue() {
        return this.value;
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        } else if (o != null && this.getClass() == o.getClass()) {
            AbstractChar that = (AbstractChar) o;
            return this.value.equals(that.value);
        } else {
            return false;
        }
    }

    public int hashCode() {
        return this.value.hashCode();
    }

    public String toString() {
        return this.value;
    }

    public int length() {
        return this.value.length();
    }
}
