package com.huawei.dli.jdbc.utils.type;

public class DecimalTypeInfo extends AbstractPrimitiveTypeInfo {

    static final int DEFAULT_PRECISION = 54;

    static final int DEFAULT_SCALE = 18;

    private int precision;

    private int scale;

    DecimalTypeInfo() {
        this(DEFAULT_PRECISION, DEFAULT_SCALE);
    }

    DecimalTypeInfo(int precision, int scale) {
        super(DliType.DECIMAL);
        this.validateParameter(precision, scale);
        this.precision = precision;
        this.scale = scale;
    }

    private void validateParameter(int precision, int scale) {
        if (precision >= 1 && precision <= 54) {
            if (scale >= 0 && scale <= 18) {
                if (scale > precision) {
                    throw new IllegalArgumentException("Decimal precision must be larger than or equal to scale");
                }
            } else {
                throw new IllegalArgumentException("Decimal scale " + scale + " out of range: [0, " + 18 + "]");
            }
        } else {
            throw new IllegalArgumentException("Decimal precision " + precision + " out of range: [1, " + 54 + "]");
        }
    }

    public String getTypeName() {
        return this.precision == 54 && this.scale == 18
            ? super.getTypeName() : String.format("%s(%s,%s)", super.getTypeName(), this.precision, this.scale);
    }

    public int getPrecision() {
        return this.precision;
    }

    public int getScale() {
        return this.scale;
    }

}
