package com.huawei.dli.jdbc.utils.type;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

@SuppressWarnings("unchecked")
public class DliTypeTransformer {

    private static final int STRING_MAX_LENGTH = 8388608;

    private static final long DATETIME_MAX_TICKS = 253402271999000L;

    private static final long DATETIME_MIN_TICKS = -62135798400000L;

    private static Map<DliType, Class> dliTypeMapper = new HashMap();

    DliTypeTransformer() {
    }

    public static Class dliTypeToJavaType(DliType type) {
        if (dliTypeMapper.containsKey(type)) {
            return dliTypeMapper.get(type);
        } else {
            throw new IllegalArgumentException("Cannot get Java type for DLI type: " + type);
        }
    }

    static {
        dliTypeMapper.put(DliType.BIGINT, Long.class);
        dliTypeMapper.put(DliType.LONG, Long.class);
        dliTypeMapper.put(DliType.STRING, String.class);
        dliTypeMapper.put(DliType.DOUBLE, Double.class);
        dliTypeMapper.put(DliType.INT, Integer.class);
        dliTypeMapper.put(DliType.SMALLINT, Short.class);
        dliTypeMapper.put(DliType.DATE, java.sql.Date.class);
        dliTypeMapper.put(DliType.TIMESTAMP, Timestamp.class);
        dliTypeMapper.put(DliType.CHAR, Char.class);
        dliTypeMapper.put(DliType.VARCHAR, Varchar.class);
        dliTypeMapper.put(DliType.SHORT, Short.class);
        dliTypeMapper.put(DliType.DECIMAL, BigDecimal.class);
        dliTypeMapper.put(DliType.FLOAT, Float.class);
        dliTypeMapper.put(DliType.TINYINT, Byte.class);
        dliTypeMapper.put(DliType.ARRAY, java.sql.Array.class);
        dliTypeMapper.put(DliType.BOOLEAN, Boolean.class);
        dliTypeMapper.put(DliType.STRUCT, java.sql.Struct.class);
        dliTypeMapper.put(DliType.MAP, String.class);
    }
}
