package com.huawei.dli.jdbc.utils.type;

import com.google.gson.JsonArray;
import com.google.gson.JsonParser;

import java.sql.Array;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SimpleArray implements Array {
    protected ArrayTypeInfo typeInfo;

    protected List<Object> values;

    protected Object objValue;

    public SimpleArray(ArrayTypeInfo typeInfo, Object obj) {
        if (typeInfo == null) {
            throw new IllegalArgumentException("Illegal arguments for StructObject.");
        }
        this.typeInfo = typeInfo;
        this.objValue = obj;
    }

    public String toString() {
        return this.objValue.toString();
    }

    @Override
    public String getBaseTypeName() throws SQLException {
        return DliType.ARRAY.name();
    }

    @Override
    public int getBaseType() throws SQLException {
        return java.sql.Types.ARRAY;
    }

    @Override
    public Object getArray() throws SQLException {
        if (values == null) {
            parseArray();
        }
        return values.toArray();
    }

    private void parseArray() throws SQLException {
        JsonArray jsonArray = JsonParser.parseString(objValue.toString()).getAsJsonArray();
        TypeInfo elementTypeInfo = typeInfo.getElementTypeInfo();
        this.values = new ArrayList<>();

        for (Object obj : jsonArray) {
            this.values.add(TypeUtils.getField(obj, elementTypeInfo));
        }
    }

    @Override
    public Object getArray(Map<String, Class<?>> map) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public Object getArray(long index, int count) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public Object getArray(long index, int count, Map<String, Class<?>> map) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public ResultSet getResultSet() throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public ResultSet getResultSet(Map<String, Class<?>> map) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public ResultSet getResultSet(long index, int count) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public ResultSet getResultSet(long index, int count, Map<String, Class<?>> map) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }

    @Override
    public void free() throws SQLException {

    }
}
