package com.huawei.dli.jdbc.utils.type;

import com.google.gson.JsonElement;
import com.google.gson.JsonNull;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import java.sql.SQLException;
import java.sql.SQLFeatureNotSupportedException;
import java.sql.Struct;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class SimpleStruct implements Struct {
    protected StructTypeInfo typeInfo;

    protected List<Object> values;

    protected Object objValue;

    public SimpleStruct(StructTypeInfo typeInfo, Object obj) {
        if (typeInfo == null) {
            throw new IllegalArgumentException("Illegal arguments for StructObject.");
        }
        this.typeInfo = typeInfo;
        this.objValue = obj;
    }

    private void parseStruct() throws SQLException {
        values = new ArrayList<>();
        if (objValue.toString().equalsIgnoreCase("null")) {
            values.add(null);
            return;
        }
        JsonObject jsonObj = JsonParser.parseString(objValue.toString()).getAsJsonObject();

        int fieldIdx = 0;
        for (String fieldName : typeInfo.getFieldNames()) {
            JsonElement fieldObj = jsonObj.get(fieldName);
            TypeInfo fieldTypeInfo = typeInfo.getFieldTypeInfos().get(fieldIdx);

            values.add(TypeUtils.getField(
                    fieldObj instanceof JsonNull ? null : fieldObj.getAsString(), fieldTypeInfo));
            fieldIdx++;
        }
    }

    public String toString() {
        return this.objValue.toString();
    }

    @Override
    public String getSQLTypeName() throws SQLException {
        return DliType.STRUCT.name();
    }

    @Override
    public Object[] getAttributes() throws SQLException {
        if (values == null) {
            parseStruct();
        }
        return values.toArray();
    }

    @Override
    public Object[] getAttributes(Map<String, Class<?>> map) throws SQLException {
        throw new SQLFeatureNotSupportedException();
    }
}
