package com.huawei.dli.jdbc.utils.type;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class SimpleStructTypeInfo implements StructTypeInfo {
    private List<String> fieldNames;

    private ArrayList<TypeInfo> fieldTypeInfos;

    SimpleStructTypeInfo(List<String> names, List<TypeInfo> typeInfos) {
        this.validateParameters(names, typeInfos);
        this.fieldNames = this.toLowerCase(names);
        this.fieldTypeInfos = new ArrayList<>(typeInfos);
    }

    private List<String> toLowerCase(List<String> names) {
        List<String> lowerNames = new ArrayList<String>(names.size());

        for (String name : names) {
            lowerNames.add(name.toLowerCase(Locale.US));
        }

        return lowerNames;
    }

    private void validateParameters(List<String> names, List<TypeInfo> typeInfos) {
        if (names != null && typeInfos != null && !names.isEmpty() && !typeInfos.isEmpty()) {
            if (names.size() != typeInfos.size()) {
                throw new IllegalArgumentException(
                    "The amount of field names must be equal to the amount of field types.");
            }
        } else {
            throw new IllegalArgumentException("Invalid name or element type for struct.");
        }
    }

    @Override
    public String getTypeName() {
        StringBuilder stringBuilder = new StringBuilder(getDliType().name());
        stringBuilder.append("<");

        for (int i = 0; i < fieldNames.size(); ++i) {
            if (i > 0) {
                stringBuilder.append(",");
            }
            stringBuilder.append(fieldNames.get(i));
            stringBuilder.append(":");
            stringBuilder.append(fieldTypeInfos.get(i).getTypeName());
        }

        stringBuilder.append(">");

        return stringBuilder.toString();
    }

    @Override
    public DliType getDliType() {
        return DliType.STRUCT;
    }

    @Override
    public List<String> getFieldNames() {
        return fieldNames;
    }

    @Override
    public List<TypeInfo> getFieldTypeInfos() {
        return fieldTypeInfos;
    }

    @Override
    public int getFieldCount() {
        return fieldNames.size();
    }
}
