package com.huawei.dli.jdbc.utils.type;

import java.util.ArrayList;
import java.util.HashMap;

public class TypeInfoFactory {
    public static final PrimitiveTypeInfo SMALLINT;

    public static final PrimitiveTypeInfo INT;

    public static final PrimitiveTypeInfo BIGINT;

    public static final PrimitiveTypeInfo DOUBLE;

    public static final PrimitiveTypeInfo STRING;

    public static final PrimitiveTypeInfo DATE;

    public static final PrimitiveTypeInfo TIMESTAMP;

    public static final PrimitiveTypeInfo SHORT;

    public static final PrimitiveTypeInfo LONG;

    public static final PrimitiveTypeInfo BOOLEAN;

    public static final DecimalTypeInfo DECIMAL;

    public static final PrimitiveTypeInfo FLOAT;

    public static final PrimitiveTypeInfo TINYINT;

    public static final PrimitiveTypeInfo NULL;

    public static final PrimitiveTypeInfo VARCHAR;

    public static final PrimitiveTypeInfo CHAR;

    private static HashMap<DliType, PrimitiveTypeInfo> typeInfoMap;

    private TypeInfoFactory() {
    }

    public static PrimitiveTypeInfo getPrimitiveTypeInfo(DliType dliType) {
        PrimitiveTypeInfo typeInfo = (PrimitiveTypeInfo) typeInfoMap.get(dliType);
        if (typeInfo != null) {
            return typeInfo;
        } else {
            throw new IllegalArgumentException("Error get PrimitiveTypeInfo instance for: " + dliType);
        }
    }

    public static CharTypeInfo getCharTypeInfo(int length) {
        return new CharTypeInfo(length);
    }

    public static VarcharTypeInfo getVarcharTypeInfo(int length) {
        return new VarcharTypeInfo(length);
    }

    public static DecimalTypeInfo getDecimalTypeInfo(int precision, int scale) {
        return precision == DecimalTypeInfo.DEFAULT_PRECISION && scale == DecimalTypeInfo.DEFAULT_SCALE
            ? DECIMAL : new DecimalTypeInfo(precision, scale);
    }

    public static ArrayTypeInfo getArrayTypeInfo(TypeInfo valueType) {
        return new SimpleArrayTypeInfo(valueType);
    }

    public static StructTypeInfo getStructTypeInfo(ArrayList<String> names,
        ArrayList<TypeInfo> typeInfos) {
        return new SimpleStructTypeInfo(names, typeInfos);
    }

    public static MapTypeInfo getMapTypeInfo(TypeInfo keyType, TypeInfo valueType) {
        return new SimpleMapTypeInfo(keyType, valueType);
    }

    static {
        SMALLINT = new SimplePrimitiveTypeInfo(DliType.SMALLINT);
        INT = new SimplePrimitiveTypeInfo(DliType.INT);
        BIGINT = new SimplePrimitiveTypeInfo(DliType.BIGINT);
        DOUBLE = new SimplePrimitiveTypeInfo(DliType.DOUBLE);
        STRING = new SimplePrimitiveTypeInfo(DliType.STRING);
        DATE = new SimplePrimitiveTypeInfo(DliType.DATE);
        TIMESTAMP = new SimplePrimitiveTypeInfo(DliType.TIMESTAMP);
        SHORT = new SimplePrimitiveTypeInfo(DliType.SHORT);
        LONG = new SimplePrimitiveTypeInfo(DliType.LONG);
        BOOLEAN = new SimplePrimitiveTypeInfo(DliType.BOOLEAN);
        DECIMAL = new DecimalTypeInfo();
        FLOAT = new SimplePrimitiveTypeInfo(DliType.FLOAT);
        TINYINT = new SimplePrimitiveTypeInfo(DliType.TINYINT);
        NULL = new SimplePrimitiveTypeInfo(DliType.NULL);
        VARCHAR = new SimplePrimitiveTypeInfo(DliType.VARCHAR);
        CHAR = new SimplePrimitiveTypeInfo(DliType.CHAR);

        typeInfoMap = new HashMap<>();
        typeInfoMap.put(SMALLINT.getDliType(), SMALLINT);
        typeInfoMap.put(INT.getDliType(), INT);
        typeInfoMap.put(BIGINT.getDliType(), BIGINT);
        typeInfoMap.put(DOUBLE.getDliType(), DOUBLE);
        typeInfoMap.put(STRING.getDliType(), STRING);
        typeInfoMap.put(DATE.getDliType(), DATE);
        typeInfoMap.put(BOOLEAN.getDliType(), BOOLEAN);
        typeInfoMap.put(TIMESTAMP.getDliType(), TIMESTAMP);
        typeInfoMap.put(DECIMAL.getDliType(), DECIMAL);
        typeInfoMap.put(FLOAT.getDliType(), FLOAT);
        typeInfoMap.put(TINYINT.getDliType(), TINYINT);
        typeInfoMap.put(NULL.getDliType(), NULL);
        typeInfoMap.put(VARCHAR.getDliType(), VARCHAR);
        typeInfoMap.put(CHAR.getDliType(), CHAR);
    }
}
