package com.huawei.dli.jdbc.utils.type;

import com.huawei.dli.jdbc.utils.DliColumn;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.SimpleDateFormat;

public class TypeUtils {
    public static int getIntField(Object fieldObj) throws SQLException {
        try {
            if (fieldObj instanceof Number) {
                return ((Number) fieldObj).intValue();
            } else if (fieldObj instanceof String) {
                return Integer.parseInt((String) fieldObj);
            } else if (fieldObj == null) {
                return 0;
            }
            throw new SQLException("Illegal conversion");
        } catch (Exception e) {
            throw new SQLException("Cannot convert the field " + e.toString(), e);
        }
    }

    public static long getLongField(Object obj) throws SQLException {
        try {
            if (obj instanceof Number) {
                return ((Number) obj).longValue();
            } else if (obj == null) {
                return 0;
            } else if (obj instanceof String) {
                return Long.parseLong((String) obj);
            }
            throw new SQLException("Illegal conversion");
        } catch (Exception e) {
            throw new SQLException("Cannot convert field" + e.toString(), e);
        }
    }

    public static short getShortField(Object obj) throws SQLException {
        try {
            if (obj instanceof Number) {
                return ((Number) obj).shortValue();
            } else if (obj == null) {
                return 0;
            } else if (obj instanceof String) {
                return Short.parseShort((String) obj);
            }
            throw new SQLException("Illegal conversion");
        } catch (Exception e) {
            throw new SQLException("Cannot convert field" + e.toString(), e);
        }
    }

    public static double getDoubleField(Object obj) throws SQLException {
        try {
            if (obj instanceof Number) {
                return ((Number) obj).doubleValue();
            } else if (obj == null) {
                return 0;
            } else if (obj instanceof String) {
                return Double.parseDouble((String) obj);
            }
            throw new SQLException("Illegal conversion");
        } catch (Exception e) {
            throw new SQLException("Cannot convert field" + e.toString(), e);
        }
    }

    public static String getStringField(Object obj) throws SQLException {
        if (obj == null) {
            return null;
        } else if (obj instanceof byte[]) {
            return new String((byte[]) obj);
        } else if (obj instanceof java.util.Date) {
            SimpleDateFormat dateFormat = new SimpleDateFormat(DliColumn.DLI_DATETIME_FORMAT);
            return dateFormat.format(((java.util.Date) obj));
        } else {
            return obj.toString();
        }
    }

    public static boolean getBooleanField(Object obj) throws SQLException {
        if (obj instanceof Boolean) {
            return (Boolean) obj;
        } else if (obj == null) {
            return false;
        } else if (obj instanceof Number) {
            return ((Number) obj).intValue() != 0;
        } else if (obj instanceof String) {
            return !obj.equals("0");
        }
        throw new SQLException("Cannot convert column to boolean");
    }

    public static BigDecimal getBigDecimalField(Object obj) throws SQLException {
        return BigDecimal.valueOf(getLongField(obj));
    }

    public static Object getField(Object fieldObj, TypeInfo fieldTypeInfo) throws SQLException {
        switch (fieldTypeInfo.getDliType()) {
            case INT:
                return TypeUtils.getIntField(fieldObj);
            case BIGINT:
                return TypeUtils.getLongField(fieldObj);
            case SMALLINT:
            case TINYINT:
            case SHORT:
                return TypeUtils.getShortField(fieldObj);
            case DOUBLE:
            case FLOAT:
                return TypeUtils.getDoubleField(fieldObj);
            case STRING:
            case CHAR:
            case VARCHAR:
                return TypeUtils.getStringField(fieldObj);
            case BOOLEAN:
                return TypeUtils.getBooleanField(fieldObj);
            case DECIMAL:
                return TypeUtils.getBigDecimalField(fieldObj);
            case STRUCT:
                return new SimpleStruct((StructTypeInfo) fieldTypeInfo, fieldObj);
            default:
                return fieldObj;
        }
    }
}
